@@ -1,5 +1,5 @@
 /*
- *  $Id: dialog.h,v 1.192 2007/02/23 22:47:13 tom Exp $
+ *  $Id: dialog.h,v 1.195 2008/03/16 19:14:14 tom Exp $
  *
  * dialog.h -- common declarations for all dialog modules
  *
@@ -421,10 +421,12 @@ typedef struct {
     unsigned input_length;	/* nonzero if input_result is allocated */
     /* 1.0-20051207 */
     unsigned formitem_type;	/* DIALOG_FORMITEM.type in dialog_form() */
-    /* 1.0-20070227 */
+    /* 1.1-20070227 */
     bool keep_tite;		/* option "--keep-tite" */
     bool ascii_lines;		/* option "--ascii-lines" */
     bool no_lines;		/* option "--no-lines" */
+    /* 1.1-20070930 */
+    bool nook;			/* option "--no-ok" */
 } DIALOG_VARS;
 
 #define USE_ITEM_HELP(s)        (dialog_vars.item_help && (s) != 0)
@@ -583,6 +585,7 @@ extern void dlg_remove_callback(DIALOG_CALLBACK * /*p*/);
 extern void dlg_killall_bg(int *retval);
 
 /* util.c */
+extern WINDOW * dlg_new_modal_window(WINDOW * /*parent*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern WINDOW * dlg_new_window(int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern WINDOW * dlg_sub_window(WINDOW * /*win*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern char * dlg_set_result(const char * /*string*/);
@@ -646,10 +649,17 @@ extern int dlg_strcmp(const char * /*a*/, const char * /*b*/);
 #endif
 
 #ifdef HAVE_DLG_TRACE
+#define DLG_TRACE(params) dlg_trace_msg params
+extern void dlg_trace_msg(const char *fmt, ...)
+#ifdef GCC_PRINTF
+    __attribute__((format(printf,1,2)))
+#endif
+;
 extern void dlg_trace_win(WINDOW * /*win*/);
 extern void dlg_trace_chr(int /*ch*/, int /*fkey*/);
 extern void dlg_trace(const char * /*fname*/);
 #else
+#define DLG_TRACE(params) /* nothing */
 #define dlg_trace_win(win) /* nothing */
 #define dlg_trace_chr(ch,fkey) /* nothing */
 #define dlg_trace(fname) /* nothing */
